#!/usr/bin/env python
from sallron.util import settings
from sallron.db import mongo
from walrus import Database
from redis import Redis
from time import sleep
import datetime
import schedule
import logging
import os
import sys

_REDIS_HOST = '127.0.0.1' # standard

class RedisCache():
    def __init__(self):

        self.cache_logger = logging.getLogger('cache')
        self.cache = Database().cache()

        self.connection_retries = 0
        self.max_connection_retries = 3

        # redis-server check and handling
        if not(self._check_connection()):
            self.cache_logger.warning("Redis-server not running!")
            try:
                self.cache_logger.info("Starting redis-server on background.")
                self._start_redis()
            except:
                # self.cache_logger.warning("Redis-server not installed!")
                # self._install_redis()
                self.cache_logger.warning("Redis-server could not be started!")
                sys.exit()

    def schedule_new_customers_checking(self, mongo_client, interface, interface_name, fn):
        """
        Utility function to schedule checking of customers

        Args:
            mongo_client (MongoDB class instance): MongoDB class instance defined in ```e_raptor/db/mongo.py```
            interface (str): Interface that is running at the moment.
            fn (function object): Function to be scheduled in case a new customer was added.
        """
        self.cache.set(f"{interface_name}_customers", mongo_client.databases)

        # schedule checking new customers
        schedule.every().day.at("15:00").do(
            self._check_customers,
            interface=interface,
            interface_name=interface_name,
            fn=fn
        )

    def _check_customers(self, interface, interface_name, fn):
        """
        Utility function to check whether a new customer was added to MongoDB.
        In case it was, schedule all functions to the new customer

        Args:
            interface (str): Interface that is running at the moment.
            fn (function object): Function to be scheduled in case a new customer was added.
        """

        current_customers = self.cache.get(f"{interface_name}_customers")

        real_customers = mongo.MongoDB().databases

        intersection = list(
            set(current_customers).intersection(real_customers)
        )

        if intersection == real_customers:

            self.cache_logger.info("No new customer added between {} and {}".format(
                self.cache.get("lastRun"),
                datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            ))
        
        else:

            self.cache.set(f"{interface_name}_customers", real_customers)

            for customer in set(real_customers) - set(current_customers):

                fn(customer, interface, interface_name)
                self.cache_logger.info("New customer added between {} and {}: {}".format(
                    self.cache.get("lastRun"),
                    datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                    customer
                ))

            schedule.run_all()

        # set date of last time this function has run
        self.cache.set("lastRun", datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

    def _check_connection(self):
        r = Redis(_REDIS_HOST, socket_connect_timeout=1) # short timeout for the test
        try:
            r.ping() 
            self.cache_logger.info('Connected to Redis @"{}"'.format(_REDIS_HOST))
            return True
        except:
            self.cache_logger.info('Failed to ping Redis, check your connection.')
            return False

    def _start_redis(self):
        if settings.OS == 'UBUNTU':
            os.popen("service redis-server start --daemonize yes")
        else:
            os.popen("redis-server --daemonize yes")
        sleep(2)
        if (self._check_connection()):
            self.cache_logger.info("Redis-server started.")
            self.connection_retries = 0
        else:
            self.connection_retries += self.connection_retries
            if self.connection_retries < self.max_connection_retries:
                self.cache_logger.info(f"Failed to start redis-server, retrying for the {self.connection_retries} time.")
                self._start_redis()
            else:
                self.cache_logger.warning("Max retries reached, check your redis-server installation/config.")

    def _install_redis(self):
        """
        Try to stay away from using this function. Not tested.
        """
        os.popen("sudo -s")
        if settings.OS == 'UBUNTU':
            os.popen("apt install redis-server")
        else:
            os.popen("brew install redis")
        sleep(10)
        self.cache_logger.info("Redis-server installed.")



import time
import sys
from subprocess import Popen
from os.path import join, dirname, exists

_KILL_TIMEOUT = 3
_PID_FILE = 'sallron.pid'

def send_message():
    """
    To be implemented.
    The idea here is:
        1. overwrite standard exception flow with custom exception_logger found in util/logger.py
        2. send message with resulting logs on every restart
        3. accompany message with deploy + customer name
    """
    if exists(_PID_FILE):
        print("sallron had to be restarted, take a look at the error logs...")
    pass

def write_pid(pid):
    with open(_PID_FILE, 'w') as w:
        w.write(pid)

def kill_pid(delete_file=True):
    '''Kill processes.'''
    with open(_PID_FILE, 'r') as rapid:
        pid = rapid.read()
    if delete_file:
        Popen(f'rm {_PID_FILE}', shell=True)
        print('.pid file purged')
    Popen('kill %s' % pid, shell=True)
    print('pid killed')

def eternal_runner(filepath):
    while True:
        try:
            send_message()
            p = Popen(f"python3 {filepath}", shell=True)
            p.wait()
        except (KeyboardInterrupt):
            print('Exit signal received, shutting down.')
            time.sleep(_KILL_TIMEOUT)
            kill_pid()
            sys.exit()
            print("That's all folks.")


#!/usr/bin/env python

from logging.handlers import RotatingFileHandler
from discord_logger import DiscordLogger
from sallron.util import settings
from datetime import datetime
import traceback
import logging
import sys
import os

def setup_logger(level = logging.INFO):
    '''Prints logger info to terminal'''
    logger = logging.getLogger()
    logger.setLevel(level)
    ch = logging.StreamHandler()
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger

def setup_logbook(name, extension='.txt', level=logging.INFO):
    """Setup logger that writes to file, supports multiple instances with no overlap.
       Available levels: DEBUG|INFO|WARN|ERROR"""
    formatter = logging.Formatter(fmt='%(asctime)s.%(msecs)03d (%(name)s) - %(message)s', datefmt='%d-%m-%y %H:%M:%S')
    date = datetime.today().strftime('%Y-%m-%d')
    log_path = str(settings.LOG_DIR + name +'_' + date + extension)
    try:
        handler = RotatingFileHandler(log_path, maxBytes=settings.MAX_LOG_SIZE, backupCount=1)
    except FileNotFoundError:
        os.mkdir(settings.LOG_DIR)
        handler = RotatingFileHandler(log_path, maxBytes=settings.MAX_LOG_SIZE, backupCount=1)
    handler.setFormatter(formatter)
    logger = logging.getLogger(name)
    logger.setLevel(level)
    logger.addHandler(handler)
    return logger

# Global error_logger so all functions can use it
error_logger = logging.getLogger('errors')

def setup_error_logger():
    '''Basic handler setup for error_logger'''
    date = datetime.today().strftime('%Y-%m-%d')
    log_path = settings.LOG_DIR + 'errors_' + date + '.txt'

    global error_logger
    error_logger = logging.getLogger('errors')
    handler = logging.handlers.RotatingFileHandler(log_path, maxBytes=settings.MAX_LOG_SIZE, backupCount=1) # stream=sys.stdout
    error_logger.addHandler(handler)

def close_error_logger():
    error_logger.removeHandler(error_logger.handlers[0])
    return

def log_exception(exc_type, exc_value, exc_traceback):
    '''Log unhandled exceptions
    set sys.excepthook = logger.log_exception on main file for it to work'''
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return
    
    error_logger.info('-----------------------------------------------------------------')
    error_logger.info(str(datetime.now()))
    error_logger.error("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))
    error_logger.info('-----------------------------------------------------------------')
    error_logger.info('')
    message = ''.join(str(exc_type) + str(exc_value)).join(traceback.format_tb(exc_traceback))
    return

def log_error(message):
    '''Log handled errors'''
    error_logger.info('-----------------------------------------------------------------')
    error_logger.info(str(datetime.now()))
    error_logger.error(message)
    error_logger.info('-----------------------------------------------------------------')
    error_logger.info('')
    return

def send_message_discord(message, metadata={}, interface_name="ga", level="error", title="Exception raised", description=" "):
    """
    Sends a message to a discord channel.

    Args:
        message (string): the message to be sent.
        metadata (dictionary): Any dictionary with information.
        interface_name (string): The name of the interface
        level (string): The level of the information being sent. Default is "error". Possibilities: error, warn, info, verbose, debug, success.
        title (string): Title of the message.
        description (string): Description of the message.
    """    

    options = {
        "application_name": "Raptor",
        "service_name": f"{interface_name}",
        "service_environment": "Production",
        "default_level": "info",
    }

    logger = DiscordLogger(webhook_url=settings._WEBHOOK, **options)
    logger.construct(
        title=f"{title}",
        level=f"{level}",
        description=f"{description}",
        error=message,
        metadata=metadata,
    )

    response = logger.send()

    pass


import requests
import functools

def get_vtex_categories(
    customer,
    product_name,
    environment="vtexcommercestable"):
    """
    This function will get the categories related to a specific product.

    Args:
        customer (str): Customer name which has the product
        product_name (str): Product name as it comes from the ga_wrapper functions 
                            access_block, exit_rate_block and sales_block.
        environment (str): Environment to use
    
    Returns: 
        unique_categories (list): A list of the categories names.
    """

    product_name = product_name.replace(" ", "-").lower()

    endpoint = f"https://{customer}.{environment}.com.br/api/catalog_system/pub/products/search/{product_name}/p"

    r = requests.get(endpoint).json()

    categories = r[0].get('categories')

    categories = list(set(functools.reduce(lambda x, y: x + y, map(
            lambda string: list(filter(
                lambda element: element,
                string.split('/')
            )),
            categories
        ))))

    return categories

def get_pagespeed_required_urls(customer, environment="vtexcommercestable"):
    """
    """
    endpoint = f"https://{customer}.{environment}.com.br/api/catalog_system/pub/products/search?fq=isAvailablePerSalesChannel_1:1&_from=1&_to=3&O=OrderByTopSaleDESC"

    r = requests.get(endpoint).json()

    productids = []
    product_text_links = []
    categories = []
    for item in r:
        productids.append(item.get('productId'))
        product_text_links.append(item.get('linkText'))
        categories.append(item.get('categories')[0])

    url_checkout = f"https://{customer}.{environment}.com.br/checkout/cart/add?sku={productids[0]}&qty=1&seller=1&sku={productids[1]}&qty=1&seller=1&sku={productids[2]}&qty=1&seller=1&redirect=true&sc=1"
    
    return url_checkout, product_text_links, categories


"""Core module for AWS S3 related operations"""
from sallron.util import settings
import pickle
import boto3
import os

def send_to_s3(key_path, obj_path):
    """
    Utility function to send objects to S3

    Args:
        key_path (str): Path to AWS keys
        obj_path (str): Path to object to be sent
    """

    # ensure everything needed is set
    if key_path and obj_path and settings.LOGGING_BUCKET:

        with open(key_path, "rb") as f:
            keys = pickle.load(f)

        s3 = boto3.session.Session(
            aws_access_key_id=keys.get("aws_access_key_id"),
            aws_secret_access_key=keys.get("aws_secret_access_key_id"),
            region_name="us-east-2"
        ).resource("s3")

        bucket = s3.bucket(settings.LOGGING_BUCKET)

        # note it expects a obj_path following path/to/obj.txt format
        # gets the last name of this sequence
        bucket.upload_file(obj_path, obj_path.split('/')[-1])

        # remove the current log file
        os.remove(obj_path)

    else:
        
        pass


#!/usr/bin/env python

MONGO_CONN_STR = "mongodb+srv://..."

ADMIN_COLLECTION = "info" # collection containing credentials and function parameters

ADMIN_DATABASES = ["admin", "local"] # databases to ignore when fetching customers

_WEBHOOK = ""

OS = 'UBUNTU'
# OS = 'MAC'

SAVE_LOGS = False

LOG_DIR = "logs/"

MAX_LOG_SIZE = 100000000 # 100Mb

LOGGING_BUCKET = ""


from sallron.util import s3
import schedule
import datetime
import pytz

def send_logging_and_schedule_kill_process(key_path, log_dir, timezone):
    """
    Utility function to send the log file to S3 and kill the hole process

    Args:
        key_path (str): Path to AWS keys
        obj_path (str): Path to object to be sent
        timezone (str): Timezone string supported by pytz
    """

    schedule.every().day.at(
        datetime.time(hour=0, minute=1, tzinfo=pytz.timezone(timezone)).strftime("%H:%M")
    ).do(s3.send_to_s3, key_path=key_path, obj_path=log_dir)